function [A] = buildBlurMatrix(v, nR, nC) 

nT = size(v, 2); 

A = spalloc(nR*nC, nR*nC, 50*nR*nC); 

ridx = repmat([1:nR]', 1, nC);
cidx = repmat([1:nC], nR, 1);

mean_x = nC/2; 
mean_y = nR/2; 

% Shift the coordinate system so that the origin lies in the centre, and 
% y-axis goes up
rx = cidx - 0.5 - mean_x; 
ry = -((ridx - 0.5) - mean_y);

ax = zeros(nR, nC, nT+1); 
ay = ax;  
ax(:,:,1) = rx; 
ay(:,:,1) = ry;

p = zeros(3, nT+1);

% Numerically integrate the velocity to get the trajectories of each pixel
for t = 1:size(v,2)
    p(:,t+1) = v(:,t) + p(:,t);
    Vx = -[ay(:,:,t) - p(2,t)]*v(3,t) + v(1,t);
    Vy = [ax(:,:,t) - p(1,t)]*v(3,t) + v(2,t);
    ax(:,:,t+1) = ax(:,:,t) + Vx;
    ay(:,:,t+1) = ay(:,:,t) + Vy;
end

% Convert back to row-column space, pc and pr are stacks of column and 
% row coordinates through time
pc = ax + mean_x + 0.5;
pr = -ay + mean_y + 0.5; 


% We need the 4 pixels near by to do bilinear interpolation, alpha
% represents the mixing portion
cpc = ceil(pc);
cpr = ceil(pr);
fpc = floor(pc);
fpr = floor(pr);
alphac = pc - fpc;
alphar = pr - fpr;

% Handle image boundary by clipping the coordinates to the edges
fpr(fpr < 1) = 1;
fpr(fpr >= nR) = nR;
cpr(cpr < 1) = 1;
cpr(cpr >= nR) = nR;
fpc(fpc < 1) = 1;
fpc(fpc >= nC) = nC;
cpc(cpc < 1) = 1;
cpc(cpc >= nC) = nC;

i00 = nR*(fpc - 1) + fpr;
i01 = nR*(cpc - 1) + fpr;
i10 = nR*(fpc - 1) + cpr;
i11 = nR*(cpc - 1) + cpr;

v00 = (1 - alphac).*(1 - alphar); 
v01 = alphac.*(1 - alphar); 
v10 = (1 - alphac).*alphar; 
v11 = alphac.*alphar; 
for t = 1:nT+1
    t
    for r = 1:nR
        for c = 1:nC
            idx = nR*(c - 1) + r;
            % Enter the proper weights from bilinear interpolation inter
            % the matrix, this is where the bottleneck is. 
            A(idx, i00(r,c,t)) = A(idx, i00(r,c,t)) + v00(r,c,t); 
            A(idx, i01(r,c,t)) = A(idx, i01(r,c,t)) + v01(r,c,t);
            A(idx, i10(r,c,t)) = A(idx, i10(r,c,t)) + v10(r,c,t);
            A(idx, i11(r,c,t)) = A(idx, i11(r,c,t)) + v11(r,c,t);
        end
    end
end

A = A ./ (nT+1);
