load data; 

% E is the original image
E = imresize(E, 0.5); 

% Generate a velocity trajectory representing camera movement
v = randn(3, 20);
v(1:2,:) = v(1:2,:);
%v(1:2,:) = 0.01; 
v(3,:) = 0.01*v(3,:);  % Use a smaller variance for angular velocity 
%v(3,:) = 0; 
%v(3,:) = -0.001; 

[B ax ay] = blurImage(E, v); 
step = 50; 


figure;

% Display the original image
subplot(3,3,1); 
imshow(E); 

% Display pixel paths, representing the blur kernel
subplot(3,3,2);
hold on;
for t = 1:size(ax, 3)
    plot(ax(1:step:end, 1:step:end,t), ay(1:step:end, 1:step:end,t), 'ob'); axis equal; 
    %pause; 
end

plot(ax(1:step:end, 1:step:end,1), ay(1:step:end, 1:step:end,1), 'or');  
axis tight

% Display the blurred image
subplot(3,3,3); 
imshow(B);

% Tryout Matlab optimization to solve for the camera movement
opts = optimset('Display','iter', 'GradObj','on', 'MaxIter', 100, 'LargeScale', 'off');%, 'DerivativeCheck', 'on');
fn = @blurObj;

% Initialize the optimization
params = zeros(1,size(v,1)*size(v,2)); 
v0 = v; 
v0(1:2,:) = v(1:2,:) + randn(2,size(v0,2)); 
%v0(3,:) = v(3,:) + 0.01*randn(1,size(v0,2)); 
%v0(1:2,:) = 0; 
v0(3,:) = 0; 
params = v0(:)'; 


% Display the initialized camera movement and blurred image 
[C0 bx0 by0] = blurImage(E, v0);

subplot(3,3,4); 
imshow(E); 

subplot(3,3,5);
hold on;
for t = 1:size(ax, 3) 
    plot(bx0(1:step:end, 1:step:end,t), by0(1:step:end, 1:step:end,t), 'ob'); axis equal; 
    %pause; 
end

plot(bx0(1:step:end, 1:step:end,1), by0(1:step:end, 1:step:end,1), 'or');  
axis tight

subplot(3,3,6);
imshow(C0); 

for ii = 1:10
    % Run optimization
    [params,fx,exitflag] = fminunc(@(x)fn(x, E, B), params, opts);
    
    % Check intermediate optimization results
    u = reshape(params, 3, length(params)/3);

    [C bx by] = blurImage(E, u);

    subplot(3,3,7); 
    imshow(E); 
    
    subplot(3,3,8);
    hold off; 

    for t = 1:size(bx, 3)
        plot(bx(1:step:end, 1:step:end,t), by(1:step:end, 1:step:end,t), 'ob'); axis equal;
        if t == 1
            hold on; 
        end
        %pause;
    end

    plot(bx(1:step:end, 1:step:end,1), by(1:step:end, 1:step:end,1), 'or');
    axis tight

    subplot(3,3,9);
    imshow(C);
    %pause;
end


