load data; 

% E is the original image
E = imresize(E, 0.1); 
nT = 50; 
nR = size(E, 1); 
nC = size(E, 2); 
% Generate a velocity trajectory representing camera movement
v = randn(3, nT);
v(1:2,:) = v(1:2,:);
%v(1:2,:) = 0.01; 
v(3,:) = 0.01*v(3,:);  % Use a smaller variance for angular velocity 
%v(3,:) = 0; 
%v(3,:) = -0.001; 

[B ax ay] = blurImage(E, v);

step = 25; 

figure;

% Display the original image
subplot(1,3,1); 
imshow(E); 

% Display pixel paths, representing the blur kernel
subplot(1,3,2);
hold on;
for t = 1:size(ax, 3)
    plot(ax(1:step:end, 1:step:end,t), ay(1:step:end, 1:step:end,t), 'ob'); axis equal; 
    %pause; 
end

plot(ax(1:step:end, 1:step:end,1), ay(1:step:end, 1:step:end,1), 'or');  
axis tight

% Display the blurred image
subplot(1,3,3); 
imshow(B);


% Initial guess from matlab's motion kernel
K = fspecial('motion', 10, 15); 
vv = kernelToTraj(K, 20); 
vv = [vv; zeros(1, 20)]; 
%B = imfilter(E, K);  
vguess = vv; 


% Uncomment if we want to optimize for the trajectory first
% INITPSF = ones(7,7);
% [Eguess P] = deconvblind(B,INITPSF);
% 
% 
% % Tryout Matlab optimization to solve for the camera movement
% opts = optimset('Display','iter', 'GradObj','on', 'MaxIter', 100, 'LargeScale', 'off');%, 'DerivativeCheck', 'on');
% fn = @blurObj;
% 
% % Initialize the optimization
% params = zeros(1,size(v,1)*size(v,2));
% nT = 20;
% v0 = vv;
% params = v0(:)';
% 
% % Run optimization
% [params,fx,exitflag] = fminunc(@(x)fn(x, Eguess, B), params, opts);
% 
% % Check intermediate optimization results
% vguess = reshape(params, 3, length(params)/3);


for ii = 1:10
    figure; 
    A = buildBlurMatrix(vguess, nR, nC);
    
    % Try solving for the original image by constrained LSQ
    lb = zeros(size(B(:)));
    V = lsqlin(A, B(:), [], [], [], [], lb, []);
    Eguess = reshape(V, nR, nC);

    [C bx by] = blurImage(Eguess, vguess);

    subplot(1,3,1); 
    imshow(Eguess); 
    
    subplot(1,3,2);
    hold off; 

    for t = 1:size(bx, 3)
        plot(bx(1:step:end, 1:step:end,t), by(1:step:end, 1:step:end,t), 'ob'); axis equal;
        if t == 1
            hold on; 
        end
        %pause;
    end

    plot(bx(1:step:end, 1:step:end,1), by(1:step:end, 1:step:end,1), 'or');
    axis tight

    subplot(1,3,3);
    imshow(C);
    
    % Solve for the trajectory
    % Run optimization
    [params,fx,exitflag] = fminunc(@(x)fn(x, Eguess, B), params, opts);
    
    % Check intermediate optimization results
    vguess = reshape(params, 3, length(params)/3);
end

